--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersContracts by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.2.3.9 Compatible with Seasons and Precision Farming
-- 30.11.2018 - 02.01.2021

--[[

	Added with version 1.1.0.0

	With this script the user can set up a farming contractor service and
	supplement the income from farming with high level contracting jobs using
	your own equipment.

	This is different from the concept of "missions", where a person will go around
	and do odd jobs for a farmer, using the farmers equipment.

	Contract (mission) earnings is split into the following components:

	Start fee
	Labour
	Vehicle use cost
	Farm service

	All parameters are set in RealLifeNumbersInitalization.lua

	Example output to log file:

	sow field                                =        21
	sow fieldArea                            =        1.2233 Ha
	sow SalaryPerHa                          =        5.0000 €/Ha
	sow Start fee                            =       20.0000 €
	sow Labor total                          =        6.1164 €
	sow vehicleUseCost                       =        6.1164 €
	sow farmService                          =       48.9310 €
	sow Total                                =      111.1638 €

	SalaryPerHa is calculated from the wage and the number of hectares worked per hour.

	For sowing RN.missionPayPerHour = 10 €/hour
	For sowing RN.missionHaPerHour = 2 ha/hour
	Hence for sowing, the fee becomes 10/2 = 5€/ha

	The user can define the wages and work capacity for each contract type
	in RealLifeNumbersInitalization.lua. With respect to wages, one should
	consider how much training is needed to do the job. Handling herbicide
	requires much more training than spraying dry fertilizer, hence the
	wages is set higher for spraying than for fertilizing.

	It is up to the user to define the conditions in contracts.

	PS. FS19 gives a fraction of the payback for sold fruits to the contractor.
	The mod does not change this, as it would make the information on contracts event
	even more confusing.

--]]

RealNumbersContracts = {};

function RealNumbersContracts:loadMap()
	if RN.runContractScript then
		print("\nFS19_RealLifeNumbers: RealNumbersContracts active, press lAlt-rc (if on host PC)\n");
	else
		print("\nFS19_RealLifeNumbers: RealNumbersContracts active, but not 100% multiplayer ready\n");
	end;
	MissionManager.CATEGORY_FIELD = 1.000000;
	MissionManager.AI_PRICE_MULTIPLIER = 1.000000;
	MissionManager.MAX_MISSIONS_PER_GENERATION = 4.000000;
	MissionManager.CATEGORY_GRASS_FIELD = 3.000000;
	MissionManager.MAX_TRIES_PER_GENERATION = 5.000000;
	MissionManager.MAX_MISSIONS = 25.000000;
	MissionManager.MISSION_GENERATION_INTERVAL = 14400000.000000;
	MissionManager.CATEGORY_TRANSPORT = 2.000000;
	MissionManager.MAX_TRANSPORT_MISSIONS = 2.000000;

	AbstractFieldMission.VEHICLE_USE_COST = 10.000000;
	AbstractFieldMission.classId = 68.000000;
	AbstractFieldMission.FIELD_SIZE_LARGE = 5.000000;
	AbstractFieldMission.className = 'AbstractFieldMission';
	AbstractFieldMission.REWARD_PER_HA = 100.000000;
	AbstractFieldMission.REIMBURSEMENT_FACTOR = 0.950000;
	AbstractFieldMission.FIELD_SIZE_MEDIUM = 1.500000;
	RNS.notRunYetRealNumbersContracts = true;
	RNS.upStartRealNumbersContracts = true;

-- Contract data

	RN.contracts = {};
	RN.contracts.name 								= {};
	RN.contracts.REWARD_PER_HA 						= {};
	RN.contracts.REWARD_PER_HA_SMALL 				= {};	-- harvest
	RN.contracts.REWARD_PER_HA_WIDE 				= {};	-- harvest
	RN.contracts.REWARD_PER_HA_HAY					= {};	-- mow_bale
	RN.contracts.REWARD_PER_HA_SILAGE				= {};	-- mow_bale
	RN.contracts.REWARD_PER_METER					= {};  -- transport
	RN.contracts.REWARD_PER_OBJECT					= {};  -- transport
	RN.contracts.VEHICLE_USE_COST					= {};
	

	RN.contracts.REIMBURSEMENT_PER_HA				= {};	-- fertilize and spray

	RN.contracts.types = {};

	RN.contracts.types.mow_bale 	= "mow_bale";
	RN.contracts.types.plow 		= "plow";
	RN.contracts.types.cultivate 	= "cultivate";
	RN.contracts.types.sow 			= "sow";
	RN.contracts.types.harvest 		= "harvest";
	RN.contracts.types.weed 		= "weed";
	RN.contracts.types.spray 		= "spray";
	RN.contracts.types.fertilize 	= "fertilize";
	RN.contracts.types.transport 	= "transport";
	RN.contracts.types.snow 		= "snow";
	RN.contracts.types.straw 		= "straw";


-- mow_bale
	
	RN.contracts.name["mow_bale"]					= "mow_bale";
	RN.contracts.REWARD_PER_HA_HAY["mow_bale"]		= RN.contractRatePerHa.MOWERS + RN.contractRatePerHa.TEDDERS 
															+ RN.contractRatePerHa.WINDROWERS +	RN.contractRatePerHa["DRYGRASSROUNDBALE"];
	RN.contracts.REWARD_PER_HA_SILAGE["mow_bale"]	= RN.contractRatePerHa.MOWERS  
															+ RN.contractRatePerHa.WINDROWERS +	RN.contractRatePerHa["SILAGEROUNDBALE"];
	RN.contracts.VEHICLE_USE_COST["mow_bale"]		= RN.equipment.opAndSupplyCostPerHa["DRYGRASSROUNDBALE"] 
														+ RN.equipment.operatingCostPerHa["MOWERS"]
														+ RN.equipment.operatingCostPerHa["TEDDERS"]
														+ RN.equipment.operatingCostPerHa["WINDROWERS"];
-- plow
	
	RN.contracts.name["plow"]						= "plow";
	RN.contracts.REWARD_PER_HA["plow"]				= RN.contractRatePerHa.PLOWS;
	RN.contracts.VEHICLE_USE_COST["plow"]			= RN.equipment.operatingCostPerHa["PLOWS"];

-- cultivate
	
	RN.contracts.name["cultivate"]					= "cultivate";
	RN.contracts.REWARD_PER_HA["cultivate"]			= RN.contractRatePerHa.DISCHARROWS;
	RN.contracts.VEHICLE_USE_COST["cultivate"]			= RN.equipment.operatingCostPerHa["DISCHARROWS"];

-- sow 
	
	RN.contracts.name["sow"]						= "sow";
	RN.contracts.REWARD_PER_HA["sow"]				= RN.contractRatePerHa.SEEDERS;	
	RN.contracts.VEHICLE_USE_COST["sow"]			= RN.equipment.operatingCostPerHa["SEEDERS"];
	
-- harvest 
	
	RN.contracts.name["harvest"]					= "harvest";
	RN.contracts.REWARD_PER_HA["harvest"]			= RN.contractRatePerHa.HARVESTERS;
	RN.contracts.REWARD_PER_HA_SMALL["harvest"]		= RN.contractRatePerHa.HARVESTERS;	
	RN.contracts.REWARD_PER_HA_WIDE["harvest"]		= 0.91 * RN.contractRatePerHa.HARVESTERS;
	RN.contracts.VEHICLE_USE_COST["harvest"]			= RN.equipment.operatingCostPerHa["HARVESTERS"];

-- weed	
	
	RN.contracts.name["weed"]						= "weed";
	RN.contracts.REWARD_PER_HA["weed"]				= RN.contractRatePerHa.WEEDERS;
	RN.contracts.VEHICLE_USE_COST["weed"]			= RN.equipment.operatingCostPerHa["HARVESTERS"];
	
-- spray	
	
	RN.contracts.name["spray"]						= "spray";
	RN.contracts.REWARD_PER_HA["spray"]				= RN.contractRatePerHa.SPRAYERS;
	RN.contracts.REIMBURSEMENT_PER_HA["spray"]		= RN.sprayCostHERBICIDE;
	RN.contracts.VEHICLE_USE_COST["spray"]			= RN.equipment.operatingCostPerHa["SPRAYERS"];
	
	
-- fertilize	
	
	RN.contracts.name["fertilize"]					= "fertilize";
	RN.contracts.REWARD_PER_HA["fertilize"]			= RN.contractRatePerHa.FERTILIZERSPREADERS;
	RN.contracts.REIMBURSEMENT_PER_HA["fertilize"]		= RN.sprayCostPerHa["WHEAT"]["FERTILIZER"];
	RN.contracts.VEHICLE_USE_COST["fertilize"]			= RN.equipment.operatingCostPerHa["FERTILIZERSPREADERS"];

-- transport	
	
	RN.contracts.name["transport"]					= "transport";
	RN.contracts.REWARD_PER_METER["transport"]		= RN.missionPayPerHour[9]/30000;
	RN.contracts.REWARD_PER_OBJECT["transport"]		= RN.objectFee;

-- snow	
	
	RN.contracts.name["snow"]						= "snow";
	RN.contracts.REWARD_PER_HA["snow"]				= RN.contractRatePerHa.SNOW;

-- straw bale

	RN.contracts.name["straw"]						= "straw";
	RN.contracts.REWARD_PER_HA["straw"]				= RN.contractRatePerBale.STRAWROUNDBALE * RN.balesPerHa.STRAWROUNDBALE;
	RN.contracts.VEHICLE_USE_COST["straw"]			= RN.equipment.opAndSupplyCostPerHa["STRAWROUNDBALE"] 
														+ RN.equipment.operatingCostPerHa["WINDROWERS"];
end;


function RealNumbersContracts:keyEvent(unicode, sym, modifier, isDown)
	local textWidth = 40;
	local kmFee = 0;
	local objectFee = 0;
	local isDebug = RN.isDebug;
	local currency = "€";
	local fruitTypeIndex = 0;
	local fruitName = "";
	local myfruitType = {};
	local contractTypes = RN.contracts.types;
	local isHost = true;

	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end

	local savegameDirectory = g_currentMission.missionInfo.savegameDirectory;	-- works except when a new map is opened the first time
	if (savegameDirectory == nil) and (RN.multiPlayer) then isHost = false; end;	-- is this PC multiplayer host (true) or client (false)?


	if RN.runContractScript then		-- only run if set in RealNumbersInitialization

		if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_c) then

			if isDebug	then
				print("\nlAlt-rc pressed\n");
				print("General fees for contracts");
				print(" ");
			end;

			for k,missionType in pairs(g_missionManager.missionTypes) do
				if missionType ~= nil then
					local name = missionType.name;

					if (contractTypes[name] ~= nil) then
						if name == "transport" then
							myPrint(name.." fee per object ", RN.contracts.REWARD_PER_OBJECT[name], textWidth, currency.."/object");
						elseif name == "snow" then 
							myPrint(name.." farmServicePerHa ", RN.contracts.REWARD_PER_HA[name], textWidth, currency.."/Ha");
						elseif name == "mow_bale" then
						myPrint(name.." farmServicePerHa hay ", RN.contracts.REWARD_PER_HA_HAY[name], textWidth, currency.."/Ha");
						myPrint(name.." farmServicePerHa silage", RN.contracts.REWARD_PER_HA_SILAGE[name], textWidth, currency.."/Ha");
						else
							myPrint(name.." farmServicePerHa ", RN.contracts.REWARD_PER_HA[name], textWidth, currency.."/Ha");	
						end;
					else
						print("Contract of type "..name.." is not defined in FS19_RealLifeNumbers ")
					end;
				end;
			end;

			print(" ");
			if isDebug	then
				print("Contracts for available jobs");
				print("  ");
				print("An empty list indicates that either no contracts are defined");
				print("or that you are not the host of a multiplayer game");
				print("  ");
			end;

--			print_r(RN.missions);

			if isHost then
				for k,mission in pairs(g_missionManager.missions) do
					if mission == nil then
						print(" g_missionManager.missions = nil ");
					else
						if mission.type == nil then
							print(" g_missionManager.missions.type = nil ");
						else
							local missiontype = mission.type;
							local name = missiontype.name;

							local payment = mission.reward;
							if name == "transport" then
								local numObjects = mission.numObjects;
								local objectFeeTotal = numObjects * RN.objectFee;
								myPrint(name.." number of objects ", numObjects,textWidth," ");
								myPrintFloat(name.." fee per object ", RN.objectFee, textWidth, currency.."/object");
								myPrintFloat(name.." total fee ", objectFeeTotal, textWidth, currency);
							elseif name == "snow" then
								myPrintFloat(name.." Farm service fee per Ha ", payment, textWidth, currency.."/Ha");

							else
								if mission.field == nil then
									print(" g_missionManager.missions.field = nil ");
								else
									local missionfield = mission.field;
									local fieldArea = missionfield.fieldArea;
									local fieldId = missionfield.fieldId;
									myPrint(name.." field ", fieldId, textWidth," ");

									if name == "plow" then
										fruitName = "noFruitName";
									elseif name == "cultivate" then
										fruitName = "noFruitName";
									else
										if mission.fruitType ~= nil then
											fruitTypeIndex = mission.fruitType;
										elseif missionfield.fruitType ~= nil then
											fruitTypeIndex = missionfield.fruitType;
										end;
										myfruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);
										if myfruitType ~= nil then
											if myfruitType.name ~= nil then fruitName = myfruitType.name; end;
										else
											fruitName = "noFruitName";
										end;
										if RN.fruits[fruitName] == nil then
											print(" multifruit type not defined in RealLifeNumbers");
										end;
										myPrint(name.." crop: "..fruitName, fruitTypeIndex, textWidth," ");
									end;

									myPrintFloat(name.." field area ", fieldArea, textWidth,"Ha");
									if mission.sprayFactor ~= nil then
										myPrint(name.." fertilizer state", mission.sprayFactor*100, textWidth,"%");
									else
										print("mission.sprayFactor not defined, can happen in multiplayer mode");
									end;
									if mission.limeFactor ~= nil then
										myPrint(name.." lime state", mission.limeFactor*100, textWidth,"%");
									else
										print("mission.limeFactor not defined, can happen in multiplayer mode");
									end;
									if mission.weedFactor ~= nil then
										myPrint(name.." herbicide state", mission.weedFactor*100, textWidth,"%");
									else
										print("mission.weedFactor not defined, can happen in multiplayer mode");
									end;
									if mission.fieldPlowFactor ~= nil then
										myPrint(name.." plow state", mission.fieldPlowFactor*100, textWidth,"%");
									else
										print("mission.fieldPlowFactor not defined, can happen in multiplayer mode");
									end;
--[[
									if mission.field.maxKnownGrowthState ~= nil then
										myPrint(name.." max known growth state", mission.field.maxKnownGrowthState, textWidth," ");
									end;
									if mission.growthState ~= nil then
										myPrint(name.." mission growth state", mission.growthState, textWidth," ");
									end;
]]--
									myPrint(name.." yield factor", RN.missions[k]["yieldFactor"]*100, textWidth,"%");
									myPrintFloat(name.." reward per ha ", mission.rewardPerHa, textWidth, currency.."/Ha");
									myPrintFloat(name.." total reward ", fieldArea * mission.rewardPerHa, textWidth, currency);
									myPrintFloat(name.." vehicle use cost ", mission.vehicleUseCost, textWidth, currency);
									if name == "sow" then
										myPrintFloat(name.." seed cost ", RN.missions[k]["seedCost"], textWidth, currency);
									end;

									if name == "spray" then myPrintFloat(name.." spray cost ", RN.missions[k]["sprayCost"], textWidth, currency); end;
									if name == "fertilize" then myPrintFloat(name.." fertilizer cost ", RN.missions[k]["fertilizerCost"], textWidth, currency); end;
									myPrintFloat(name.." total reimbursement ", mission.reward, textWidth, currency.." ");
									if name == "mow_bale" then
										print(name.." deliver to "..mission.sellPoint.stationName);
									end;
									if name == "harvest" then
										print(name.." deliver to "..mission.sellPoint.stationName);
										if (RN.fruits[fruitName] ~= nil) then
											myPrintFloat(name.." crop price/ha ", RN.cropPricePerHa[fruitName], textWidth, currency.."/Ha");
											myPrintFloat(name.." max yield/ha ", RN.cropBonusYieldLitersPerHa[fruitName], textWidth, "L/Ha");
											myPrintFloat(name.." max crop price ", fieldArea * RN.cropPricePerHa[fruitName], textWidth, currency);
											myPrintFloat(name.." max yield ", fieldArea * RN.cropBonusYieldLitersPerHa[fruitName], textWidth, "L");
											myPrintFloat(name.." expected yield ", mission.expectedLiters, textWidth, "L");
											myPrintFloat(name.." standard crop price/L ", RN.cropPricePerLiter[fruitName], textWidth, currency.."/L");

											myPrintFloat(name.." expected crop price ", RN.missions[k]["cropPrice"], textWidth, currency);
										end;
									end;
									if name == "sow" then
										myPrintFloat(name.." seed liter/ha ", RN.missions[k]["seedRatePerHa"], textWidth, "L/Ha");
										myPrintFloat(name.." seed cost/Liter ", RN.missions[k]["seedCostPerLiter"], textWidth, currency.."/L");
										myPrintFloat(name.." seed cost/ha ", RN.missions[k]["seedCostPerHa"], textWidth, currency.."/Ha");
									end;
									if name == "spray" then
										myPrintFloat(name.." spray liter/ha ", RN.missions[k]["sprayVolume"], textWidth, "L/Ha");
										myPrintFloat(name.." spray cost/Liter ", RN.missions[k]["sprayPrice"], textWidth, currency.."/L");
										myPrintFloat(name.." spray cost/ha ", RN.missions[k]["sprayPricePerHa"], textWidth, currency.."/Ha");
									end;
									if name == "fertilize" then
										myPrintFloat(name.." fertilizer liter/ha ", RN.missions[k]["fertilizerVolume"], textWidth, "L/Ha");
										myPrintFloat(name.." fertilizer cost/Liter ", RN.missions[k]["fertilizerPrice"], textWidth, currency.."/L");
										myPrintFloat(name.." fertilizer cost/ha ", RN.missions[k]["fertilizerPricePerHa"], textWidth, currency.."/Ha");
									end;
									print(" ");
								end;
							end;
						end;
					end;
				end;
			end;
		end;

--		DebugUtil.printTableRecursively(g_missionManager.missions,"_",0,3);
	end;

end;

function RealNumbersContracts:update(dt)

	local minute = g_currentMission.environment.currentMinute;


	local runMinute = minute % 10; -- remainder
	if runMinute ~= 8 then RNS.notRunYetRealNumbersContracts = true; end;
	if runMinute == 8 or RNS.upStartRealNumbersContracts then 
		if RNS.notRunYetRealNumbersContracts then

--			print("RealNumbersContracts:update"); -- comment out when not debugging
				local fruitTypeIndex = 0;
				local myfruitType = {};
				local fruitName = " ";
				local missionfield = {};
				local fieldArea = 0;
				local fieldId = 0;
				local isHost = true;

				local savegameDirectory = g_currentMission.missionInfo.savegameDirectory;	-- works except when a new map is opened the first time
				if (savegameDirectory == nil) and (RN.multiPlayer) then isHost = false; end;	-- is this PC multiplayer host (true) or client (false)?

				for k,missionType in pairs(g_missionManager.missionTypes) do
					if missionType ~= nil then
						local name = missionType.name;
						if name == "mow_bale" then
							missionType.class.REWARD_PER_HA_HAY = RN.contracts.REWARD_PER_HA_HAY["mow_bale"];
							missionType.class.REWARD_PER_HA_SILAGE  = RN.contracts.REWARD_PER_HA_SILAGE["mow_bale"];
							BaleMission.REWARD_PER_HA_HAY = RN.contracts.REWARD_PER_HA_HAY["mow_bale"];
							BaleMission.REWARD_PER_HA_SILAGE = RN.contracts.REWARD_PER_HA_SILAGE["mow_bale"];
						end;
						if name == "plow" then
							missionType.class.REWARD_PER_HA 	= RN.contracts.REWARD_PER_HA["plow"];
							PlowMission.REWARD_PER_HA 			= RN.contracts.REWARD_PER_HA["plow"];
						end;
						if name == "cultivate" then
							missionType.class.REWARD_PER_HA 	= RN.contracts.REWARD_PER_HA["cultivate"];
							CultivateMission.REWARD_PER_HA 		= RN.contracts.REWARD_PER_HA["cultivate"];
						end;
						if name == "sow" then
							missionType.class.REWARD_PER_HA 	= RN.contracts.REWARD_PER_HA["sow"];
							SowMission.REWARD_PER_HA 			= RN.contracts.REWARD_PER_HA["sow"];
						end;
						if name == "harvest" then
							HarvestMission.REWARD_PER_HA_SMALL 		= RN.contracts.REWARD_PER_HA_SMALL["harvest"];
							missionType.class.REWARD_PER_HA_SMALL  	= RN.contracts.REWARD_PER_HA_SMALL["harvest"];
							HarvestMission.REWARD_PER_HA_WIDE 		= RN.contracts.REWARD_PER_HA_WIDE["harvest"];
							missionType.class.REWARD_PER_HA_WIDE 	= RN.contracts.REWARD_PER_HA_WIDE["harvest"];
						end;
						if name == "weed" then
							missionType.class.REWARD_PER_HA 		= RN.contracts.REWARD_PER_HA["weed"];
							WeedMission.REWARD_PER_HA 				= RN.contracts.REWARD_PER_HA["weed"];
						end;
						if name == "spray" then
							missionType.class.REWARD_PER_HA 		= RN.contracts.REWARD_PER_HA["spray"];
							missionType.class.REIMBURSEMENT_PER_HA  = RN.sprayCostHERBICIDE;
							SprayMission.REWARD_PER_HA 				= RN.contracts.REWARD_PER_HA["spray"];
							SprayMission.REIMBURSEMENT_PER_HA 		= RN.sprayCostHERBICIDE;
						end;
						if name == "fertilize" then
							missionType.class.REWARD_PER_HA 		= RN.contracts.REWARD_PER_HA["fertilize"];
							missionType.class.REIMBURSEMENT_PER_HA  = RN.sprayCostPerHa["WHEAT"]["FERTILIZER"];
							FertilizeMission.REWARD_PER_HA 			= RN.contracts.REWARD_PER_HA["fertilize"];
							FertilizeMission.REIMBURSEMENT_PER_HA 	= RN.sprayCostPerHa["WHEAT"]["FERTILIZER"];
						end;
						if name == "transport" then
							missionType.class.REWARD_PER_METER  	= RN.contracts.REWARD_PER_METER["transport"];  --Hour rate divided by mean velocity (meters per hour)
							missionType.class.REWARD_PER_OBJECT 	= RN.contracts.REWARD_PER_OBJECT["transport"];
							TransportMission.REWARD_PER_METER 		= RN.contracts.REWARD_PER_METER["transport"];
							TransportMission.REWARD_PER_OBJECT 		= RN.contracts.REWARD_PER_OBJECT["transport"];
						end;
						if name == "snow" then
							missionType.class.REWARD_PER_HA 		= RN.contracts.REWARD_PER_HA["snow"];
						end;
						if name == "straw" then
							missionType.class.REWARD_PER_HA 		= RN.contracts.REWARD_PER_HA["straw"];
						end;
						
					end;
				end;
			if isHost then			-- only run if this PC is host
				for k,mission in pairs(g_missionManager.missions) do
					fruitTypeIndex = 0;
					fruitName = " ";
					local yieldFactor = 0;
					if mission ~= nil then
						RN.missions[k] = {};

						local name = mission.type.name;
						RN.missions[k]["name"] = name;
						if name == "transport" then
							local numObjects = mission.numObjects;
--							local vehicleUseCost = RN.transportHour * RN.vehicleUseCostPerHour[9];
							local objectFeeTotal = numObjects * RN.contracts.REWARD_PER_OBJECT["transport"];
--							local salary = RN.transportHour * RN.missionPayPerHour[9];
--							local totalTransportFee = RN.transportStart + vehicleUseCost + objectFeeTotal + salary;
							local totalTransportFee = objectFeeTotal;
							mission.reward = totalTransportFee;
							RN.missions[k]["reward"] = totalTransportFee;
							RN.missions[k]["numObjects"] = numObjects;
						else


							if mission.field == nil then
									print(" g_missionManager.missions.field = nil ");
							else
								missionfield = mission.field;
								fieldArea = missionfield.fieldArea;
								fieldId = missionfield.fieldId;
								if missionfield.fruitType == nil then
									fruitTypeIndex = nil;
									fruitName = "noFruitName";
								else
									fruitTypeIndex = missionfield.fruitType;
									myfruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);
								end;
								RN.missions[k]["fruitTypeIndex"] = fruitTypeIndex;
								RN.missions[k]["fieldId"] = fieldId;
								RN.missions[k]["fieldArea"] = fieldArea;
								RN.missions[k]["fruitName"] = "noFruitName";
								if mission.sprayFactor ~= nil then
									yieldFactor = yieldFactor + mission.sprayFactor * 0.5;			-- fertilizer spray: 100% = 50% yield increase
								end;
								if mission.limeFactor ~= nil then
									yieldFactor = yieldFactor + mission.limeFactor * 0.15;			-- lime spray: 100% = 15% yield increase
								end;
								if mission.weedFactor ~= nil then
									yieldFactor = yieldFactor + mission.weedFactor * 0.20;			-- weed spray: 100% = 20% yield increase
								end;
								if mission.plowFactor ~= nil then
									yieldFactor = yieldFactor + mission.fieldPlowFactor * 0.15;		-- plow state: 100% = 15% yield increase
								end;
							end;
							RN.missions[k]["sprayFactor"] = mission.sprayFactor;
							RN.missions[k]["limeFactor"] = mission.limeFactor;
							RN.missions[k]["weedFactor"] = mission.weedFactor;
							RN.missions[k]["fieldPlowFactor"] = mission.fieldPlowFactor;
							RN.missions[k]["yieldFactor"] = 1 + yieldFactor;


							if name == "mow_bale" then
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA_HAY["mow_bale"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["mow_bale"];
								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								mission.reward = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reward"] = mission.reward;
								RN.missions[k]["sellPoint"] = mission.sellPoint.stationName;
							end;
							if name == "plow" then
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA["plow"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["plow"];
								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								mission.reward = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reward"] = mission.reward;
							end;
							if name == "cultivate" then
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA["cultivate"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["cultivate"];
								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								mission.reward = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reward"] = mission.reward;
							end;
							if name == "sow" then
								if mission.fruitType == nil then
									fruitTypeIndex = missionfield.fruitType;
								else
									fruitTypeIndex = mission.fruitType;
								end;
								myfruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);

								if myfruitType.name ~= nil then
									fruitName = myfruitType.name;
								else
									fruitName = "noFruitName";
								end;
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA["sow"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["sow"];
								if (RN.fruits[fruitName] ~= nil) then	-- defined fruits
									mission.reimbursementPerHa = RN.seedCostPerHa[fruitName];
									RN.missions[k]["seedCost"] = fieldArea * RN.seedCostPerHa[fruitName];		-- add new mission variable
									RN.missions[k]["seedRatePerHa"] = RN.seedVolumeRatePerHa[fruitName];		-- add new mission variable
									RN.missions[k]["seedCostPerLiter"] = RN.seedCostPerLiter[fruitName];		-- add new mission variable
									RN.missions[k]["seedCostPerHa"] = RN.seedCostPerHa[fruitName];				-- add new mission variable
								else									-- for undefined fruits use WHEAT parameters
									mission.reimbursementPerHa = RN.seedCostPerHa["WHEAT"];
									RN.missions[k]["seedCost"] = fieldArea * RN.seedCostPerHa["WHEAT"];			-- add new mission variable
									RN.missions[k]["seedRatePerHa"] = RN.seedVolumeRatePerHa["WHEAT"];			-- add new mission variable
									RN.missions[k]["seedCostPerLiter"] = RN.seedCostPerLiter["WHEAT"];			-- add new mission variable
									RN.missions[k]["seedCostPerHa"] = RN.seedCostPerHa["WHEAT"];				-- add new mission variable
								end

								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reimbursementPerHa"] = mission.reimbursementPerHa;
								mission.reward = fieldArea * mission.rewardPerHa + RN.missions[k]["seedCost"];
								RN.missions[k]["reward"] = mission.reward;
							end;
							if name == "harvest" then
								if mission.fruitType == nil then
									fruitTypeIndex = missionfield.fruitType;
								else
									fruitTypeIndex = mission.fruitType;
								end;
								myfruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);

								if myfruitType.name ~= nil then
									fruitName = myfruitType.name;
								else
									fruitName = "noFruitName";
								end;
								mission.rewardPerHa  = RN.contracts.REWARD_PER_HA["harvest"];
								mission.REWARD_PER_HA_SMALL = RN.contracts.REWARD_PER_HA_SMALL["harvest"];
								mission.REWARD_PER_HA_WIDE = RN.contracts.REWARD_PER_HA_WIDE["harvest"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["harvest"];
								if (RN.fruits[fruitName] ~= nil) then	-- defined fruits
									mission.expectedLiters = fieldArea * RN.missions[k]["yieldFactor"] * 0.5 * RN.cropBonusYieldLitersPerHa[fruitName];
									RN.missions[k]["cropPrice"] = RN.cropPricePerLiter[fruitName] * mission.expectedLiters;
								else
									mission.expectedLiters = 0;
									RN.missions[k]["cropPrice"] = 0;
								end;

								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								RN.missions[k]["expectedLiters"] = mission.expectedLiters;
								mission.reward = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reward"] = mission.reward;
								RN.missions[k]["sellPoint"] = mission.sellPoint.stationName;
							end;
							if name == "weed" then
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA["weed"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["cultivate"];

								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								mission.reward = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reward"] = mission.reward;
							end;
							if name == "spray" then		-- Herbicide
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA["spray"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["spray"];
								mission.reimbursementPerHa  = RN.contracts.REIMBURSEMENT_PER_HA["spray"];

								SprayMission.REIMBURSEMENT_PER_HA = RN.contracts.REIMBURSEMENT_PER_HA["spray"];
								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;
								RN.missions[k]["reimbursementPerHa"] = mission.reimbursementPerHa;
								RN.missions[k]["sprayCost"] = fieldArea * RN.sprayCostHERBICIDE;
								RN.missions[k]["sprayVolume"] = RN.sprayLitersPerHa["HERBICIDE"];
								RN.missions[k]["sprayPrice"] = RN2.pricePerLiter["HERBICIDE"];
								RN.missions[k]["sprayPricePerHa"] = RN.sprayLitersPerHa.HERBICIDE * RN2.pricePerLiter["HERBICIDE"];
								mission.reward = fieldArea * mission.rewardPerHa + RN.missions[k]["sprayCost"];
								RN.missions[k]["reward"] = mission.reward;

							end;
							if name == "fertilize" then
								local sprayCostPerHa = 0;
								local sprayVolume = 0;
								if mission.fruitType ~= nil then
									fruitTypeIndex = mission.fruitType;
								end
								if missionfield.fruitType ~= nil then	
									fruitTypeIndex = missionfield.fruitType;
								end;
								if fruitTypeIndex == 0 then fruitTypeIndex = 1; end;
								myfruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);

								if myfruitType ~= nil then
									if myfruitType.name ~= nil then	fruitName = myfruitType.name; end;
								else
									fruitName = "noFruitName";
								end;
								mission.rewardPerHa = RN.contracts.REWARD_PER_HA["fertilize"];
								mission.vehicleUseCost = fieldArea * RN.contracts.VEHICLE_USE_COST["fertilize"];
								if fruitName == "noFruitName" then
									sprayCostPerHa = RN.sprayCostPerHa["WHEAT"]["FERTILIZER"];
									sprayVolume = 1000 * RN.mean1000LSprayRate["WHEAT"]["FERTILIZER"]
								else
									sprayCostPerHa = RN.sprayCostPerHa[fruitName]["FERTILIZER"];
									sprayVolume = 1000 * RN.mean1000LSprayRate[fruitName]["FERTILIZER"]
								end;
								mission.reimbursementPerHa  = sprayCostPerHa;

								RN.missions[k]["rewardPerHa"] = mission.rewardPerHa;
								RN.missions[k]["vehicleUseCost"] = mission.vehicleUseCost;
								RN.missions[k]["serviceCost"] = fieldArea * mission.rewardPerHa;

								RN.missions[k]["reimbursementPerHa"] = mission.reimbursementPerHa;
								RN.missions[k]["fertilizerCost"] = fieldArea * sprayCostPerHa;
								RN.missions[k]["fertilizerVolume"] = sprayVolume;
								RN.missions[k]["fertilizerPrice"] = RN2.pricePerLiter["FERTILIZER"];
								RN.missions[k]["fertilizerPricePerHa"] = sprayVolume * RN2.pricePerLiter["FERTILIZER"];
								mission.reward = fieldArea * mission.rewardPerHa + RN.missions[k]["fertilizerCost"];
								RN.missions[k]["reward"] = mission.reward;
							end;
							if name == "snow" then
								mission.rewardPerHa  = RN.contracts.REWARD_PER_HA["snow"];
							end;
							if name == "straw" then
								mission.rewardPerHa  = RN.contracts.REWARD_PER_HA["straw"];
							end;
						end;
					end;
				end;
			end;
		end;
		RNS.notRunYetRealNumbersContracts = false;
		RNS.upStartRealNumbersContracts = false;
	end;
end;
function RealNumbersContracts:draw()
end;
function RealNumbersContracts:deleteMap()
end;
function RealNumbersContracts:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersContracts);
