--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersCropYield by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.2.3.9 Compatible with Seasons and Precision Farming
-- 30.11.2018 - 02.01.2021


-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE
-- COW_TYPE_BRAHMAN_BROWN COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK
-- COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

-- FruitType indices

	-- WHEAT :: 1, BARLEY :: 2, OAT :: 3, COTTON :: 4, CANOLA :: 5, SUNFLOWER :: 6,
	-- SOYBEAN :: 7, MAIZE :: 8, POTATO :: 9, SUGARBEET :: 10, OILSEEDRADISH :: 11,
	-- POPLAR :: 12, GRASS :: 13, DRYGRASS :: 14, SUGARCANE :: 15, WEED :: 16


--[[

	Verion 1.1.0.0
	
	Data format has changed in order to take multifruit maps into account. It is intended, that the modifications
	can handle maps with fruit types, which are not defined in RealNumbersInitialization. For these fruits,	
	parameter values defined by the map maker are used. Often this will be clear in the log file by a larger
	yield and a larger sell price. The mod user can make an entry for the new fruit into the script  
	RealNumbersInitialization by copying the format of another fruit type and change the fruit name and 
	parameter values. Place the new fruit in alphabetic order to facilitate later search for a particular fruit.
	
	Calculated data for seed usage and predicted crop income have been merged into this script.

Crop mass density

	Crop mass density (kg/Liter) varies as a function of moisture content. Different sources will therefore
	provide slightly different numbers. The majority of the numbers listed above are taken from
	The agricultural service of the province of Alberta, Canada.

	Be aware that Farmingsimulator requires density numbers in (metric) Tonne/Liter. Hence the numbers listed
	above is divided by 1000 when assigning the value to massPerLiter.

	Crop mass density is considered independent of geographical location.

Crop Yield

	Crop yield is assumed to be dependent on geographical location.

	Numbers for crop yield are given in a number of different units.
	EUROSTATS provides yield in	multiples of 100 kg/hectare or Tonne/hectare
	US National Agricultural Statistics Service uses lb/acre, bushels/acre, cwt/acre, ton/acre.
	Conversion to Liter/ha is required and from there to Liters/square meters.
	The following conversion numbers are used:

		1 bushel = 35.239072 Liters
		1 acre = 0.404685642 hectares
		1 CWT = 45.35924 kg

	All density numbers in the cropYield list is provided in multiples of 100kg/ha. The cropDensity
	values are used to convert cropYield numbers to Liters/square meter.

	The base yield X set in the game must be calculated from the national/regional yield Y.

	The help suggests that application of lime and fertilizer will add 15% and 25% respectively. Thus,
	one application of lime and two applications of fertilizer should give 65% increase relative to base
	yield, suggesting that X + 0.65*X = Y, that is X = Y/1.65.

	However, tests suggest that this is not how it works. It appear that base yield is the minimum yield, you
	can get if you neglect to plough, remove weed, lime and fertilizer. Doing all this chores will add 100 %
	to the base yield. It seems therefore that X + 1*X = 2 X = Y, or X = Y/2. For this reason the parameter
	gameCorrection = 1/2, not 1/1.65 as one would expect.

	To include natural variations in crop yield, the yield for each crop is scaled by a randomness factor.
	The randomness factor varies within +/- two standard deviations for the given crop. Some crops
	have a small yield fluctuation, others a larger fluctuation, all dependng on the crop and geographical
	location. The randomness factor changes at each game load. It is the value at harvet, that determines the
	obtained yield. If the harvest of a field is divided between several gameplays, different yield setting will
	be encountered for each gameplay. This may be taken to represent yield variation across the field.

	Random variation of the yield can be turned off by different values of

	randomness = 0  => randomness is off
	randomness = 1  => randomness is on

	If you want to calibrate yield, this is better done without randomness in the yield parameter. After
	calibration, randomness can be turned back on.

Windrow Yield

	Windrow yield for grain crops is about 80% of grain yield measured in kg/ha.
	Straw density is set to 0.13 kg/Liter

	Look in the log file for printed output from this script.

]]--

RealNumbersCropYield = {};



function RealNumbersCropYield:loadMap(name)
    local textWidth = 50; 					-- text width in log file
    local iFillType = 0;
	local iFruitType = 0;
	local Crop = "Empty";
	local idx = 0;
	local currency = "€";
	local bonusYield = 0;
	local baseYield = 0;
	local bonusWindrow = 0;
	local gameWindrow = 0;
	local lossWindrow = 0.05;
	local lossTedding = 0.05;
	local seedRate = 0;
	local price = 0;
	local bonusScale = 1.0;						-- for adding extra yield for perfect crop management
	local massScale = 1000;						-- FS uses Tonne/Liter for mass density
	local gameCorrection = RN.fieldWorkbaseYieldAdjustment;	-- scale factor to obtain base yield from bonus yield

	local randomYieldFactor = 1;				-- default value used if reading seed number fails
	local randomness = 0; 						--RN.randomness;	-- Set to 1 to turn randomness on, 0 to turn off
												-- Randow variation is turned off by default as Seasons produces
												-- a more realistic variation in yield due to weather and crop rotation
	local playTime = nil;
	local noPloughLoss = 0;
	local noWeedingLoss = 0;
	local LimeBonus = 0;
	local FertilizerBonus = 0;
	local grain = {};
	local rowgrain = {};
	local sugarbeet = {};
	local potato = {};
	local isDebug = RN.isDebug;  						-- Change to False when no printing to log file is desired
	local isDebug2 = false;

	local gameLevelYieldCorrection = RN.gameLevel.cropYieldCorrection;
	local Fruits = RN.fruits;							-- local copy of global variable
	local cropDensity = RN.cropDensity;					-- local copy of global variable
	local cropYield = RN.cropYield;						-- local copy of global variable
	local yieldRange = RN.yieldRange;					-- local copy of global variable
	local strawDensity = RN.forageDensity.STRAW;
	local hayDensity = RN.forageDensity.DRYGRASS;
	local silageDensity = RN.forageDensity.SILAGE;
	local TSM = RN.TSM;
	local plantsPerSqm = RN.plantsPerSqm;
	local germination = RN.germination;
	local seedPricePer100kg = RN.seedPricePer100kg;
	local cropPricePer100kg = RN.cropPricePer100kg;
	
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;


	RN.bonusYieldPerHa = {};
	
-- FruitType indices

	-- WHEAT :: 1, BARLEY :: 2, OAT :: 3, COTTON :: 4, CANOLA :: 5, SUNFLOWER :: 6,
	-- SOYBEAN :: 7, MAIZE :: 8, POTATO :: 9, SUGARBEET :: 10, OILSEEDRADISH :: 11,
	-- POPLAR :: 12, GRASS :: 13, DRYGRASS :: 14, SUGARCANE :: 15, WEED :: 16

	print("\nFS19_RealLifeNumbers: RealNumbersCropYield active\n")


	local savegameDirectory = g_currentMission.missionInfo.savegameDirectory;	-- works except when a new map is opened the first time
	if savegameDirectory ~= nil then
		local careerFile = savegameDirectory.."/careerSavegame.xml";
		if isDebug2 then print("careerFile : "..tostring(careerFile)); end;
		local xmlFile = loadXMLFile("careerFile", careerFile);
		local key = string.format("careerSavegame.statistics");
		if not hasXMLProperty(xmlFile, "careerSavegame") then
			if isDebug2 then print("Does not have XMLproperty"); end;
		end;
		playTime = 1000 * getXMLFloat(xmlFile, "careerSavegame.statistics.playTime");
	end;

	if playTime == nil then
		if isDebug2 then print("playTime is nil") end;
		randomness = 0;
	else
		local randomSeednumber = math.randomseed(playTime);
		if isDebug2 then print("playTime is "..playTime/1000); end;
		if isDebug2 then print("random number is "..math.random()); end;
	end;

-- create a list of fruits on the map and save in a table called fruitList -- moved to Initialization
--[[
    if g_currentMission.fruitTypeManager.fruitTypes == nil then print("\nNo fruitTypes yet\n"); end;
	
	local fruitList = {};
	for i,v in pairs(g_currentMission.fruitTypeManager.fruitTypes) do
		if v.name ~= "SEMIDRY_GRASS_WINDROW" then
			fruitList[i] = v.name;
		end;
--		print(v.name);
	end;
	
-- sort fruitList alphabetically and save in sortedFruits	

	local sortedFruits = {};
	for i,n in pairs(fruitList) do table.insert(sortedFruits,n); end;
	table.sort(sortedFruits);
	RN.sortedFruits = sortedFruits;
]]--
	for i,fruitName in ipairs(RN.sortedFruits) do 

		if not(string.find(fruitName, "DRY") or string.find(fruitName, "WET") or string.find(fruitName, "WINDROW") or string.find(fruitName, "FERMENTED")) then
			print(fruitName);				-- fruitName are names of fruits defined on the map.
			seedName = "SEED_"..fruitName;
			print(seedName);
		else
			RN.fert1CostPerHa[fruitName] = 0; 	-- fertilizer cost for mowed crop is zero
			RN.fert2CostPerHa[fruitName] = 0;	-- fertilizer cost for mowed crop is zero
		end;

			iFillType = g_fillTypeManager:getFillTypeIndexByName(tostring(fruitName));
			iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);

			if Fruits[fruitName] == nil then 	-- for undefined new fruitTypes, read map data into mod variables
				print(fruitName.." is not defined in RealLifeNumbers or is not yet read from map when RealNumbersCropYield is running\n");
				RN.fruits[fruitName] = "not defined in RLN";
				RN.yieldRange[fruitName] = 0;
				RN.cropDensity[fruitName] = massScale * g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter;
				baseYield = g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm / gameLevelYieldCorrection;
				bonusYield = baseYield / gameCorrection;
				RN.cropYield[fruitName] = bonusYield * RN.cropDensity[fruitName]*10000/(bonusScale * randomYieldFactor * 100);
				RN.seedUsagePerSqm[fruitName] = g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm;
				RN.TSM[fruitName] = 0.;
				RN.plantsPerSqm[fruitName] = 0.;
				RN.germination[fruitName] = 1.;
				RN.seedDensity[fruitName] = RN.cropDensity[fruitName];
				RN.seedMassRatePerHa[fruitName] = 10000 * RN.seedUsagePerSqm[fruitName] * RN.seedDensity[fruitName];
				RN.cropPricePerLiter[fruitName] = g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter;
				RN.cropPricePer100kg[fruitName] = 100 * RN.cropPricePerLiter[fruitName]/RN.cropDensity[fruitName];
				RN.seedPricePer100kg[fruitName] = 10 * RN.cropPricePer100kg[fruitName];  -- 1.5
				RN.seedCostPerHa[fruitName] = RN.seedPricePer100kg[fruitName] * RN.seedMassRatePerHa[fruitName]/100;
				RN.seederCalibration[fruitName] = 1; 
				RN.Nitrogen[fruitName]  = 180;
				RN.Phosphate[fruitName] =  45;
				RN.Potassium[fruitName] =  80;
				RN.totalFertCostPerHa[fruitName] = 0;

--				RN.cropNPKCost[fruitName]		= RN.fertCostPerKg.N * RN.Nitrogen[fruitName]
--												+ RN.fertCostPerKg.P * RN.Phosphate[fruitName]
--												+ RN.fertCostPerKg.K * RN.Potassium[fruitName]; 


-- fertilizer
				local neededN = 0;
				local neededP = 0;
				local neededK = 0;
				
				if RN.Nitrogen[fruitName] ~= nil then
					neededN = RN.Nitrogen[fruitName];
					neededP = RN.Phosphate[fruitName];
					neededK = RN.Potassium[fruitName];
				else
					neededN = RN.Nitrogen["UNDEFINED"];
					neededP = RN.Phosphate["UNDEFINED"];
					neededK = RN.Potassium["UNDEFINED"];
				end;
--[[
				local appliedManure = neededP / RN.PAvailablePerTonne.cowMANURE; -- Ton
				local manureN = appliedManure * RN.NAvailablePerTonne.cowMANURE; -- kg
				local manureP = appliedManure * RN.PAvailablePerTonne.cowMANURE; -- kg
				local manureK = appliedManure * RN.KAvailablePerTonne.cowMANURE; -- kg
				local minFertN = math.max(neededN - manureN,0); -- kg
				local minFertP = math.max(neededP - manureP,0); -- kg
				local minFertK = math.max(neededK - manureK,0); -- kg
				local mineralNApplied = minFertN/0.27;
				local appliedManureCost = appliedManure * 10 * RN.commodityPricePer100kg.MANURE;
				local minFertNCost = minFertN * RN.fertCostPerKg.N;

				local fert1N    = RN.fert1DeliveredN[fruitName];	-- kg/Ha
				local fert1P    = RN.fert1DeliveredP[fruitName];	-- kg/Ha
				local fert1K    = RN.fert1DeliveredK[fruitName];	-- kg/Ha
				local fert2N    = RN.fert2DeliveredN[fruitName];	-- kg/Ha
				local fert2P    = RN.fert2DeliveredP[fruitName];	-- kg/Ha						
				local fert2K    = RN.fert2DeliveredK[fruitName];	-- kg/Ha
				local fert1Rate = RN.fert1SprayRate[fruitName];		-- Tonne/Ha
				local fert2Rate = RN.fert2SprayRate[fruitName];		-- Tonne/Ha
				local fertType1 = tostring(RN.fertFirstApplication[fruitName]);
				local fertType2 = tostring(RN.fertSecondApplication[fruitName]);
]]--				
				local fert1Cost = RN.fert1CostPerHa[fruitName];		-- €/Ha
				local fert2Cost = RN.fert2CostPerHa[fruitName];		-- €/Ha
				RN.cropNPKCost[fruitName] = fert1Cost + fert2Cost;


-- lime
				local appliedLime = (1/3) * RN.sprayLitersPerHa.LIME * RN.commodityDensity.LIME / 1000; -- Ton, only every third year
				local appliedLimeCost = 10 * appliedLime * RN.commodityPricePer100kg.LIME;
				RN.cropLimeCost[fruitName] = appliedLimeCost;
				
--				RN.cropNPKCost[fruitName] = appliedManureCost + minFertNCost;
--				RN.cropNPKCost[fruitName] = RN.totalFertCostPerHa[fruitName];
				
				RN.cropProtectionCost[fruitName] = RN.cropProtectionCost.WHEAT;		-- default value
								
				RN.growthDays[fruitName] = 180;		-- default value
				RN.seedVolumeRatePerHa[fruitName] = RN.seedMassRatePerHa[fruitName]/RN.seedDensity[fruitName];
				RN.seedCostPerLiter[fruitName] = RN.seedDensity[fruitName] * RN.seedPricePer100kg[fruitName]/100;
			end;
			if (Fruits[fruitName] ~= nil) then 		-- defined fruits
				if RN.TSM[fruitName] > 0 then
					RN.seedMassRatePerHa[fruitName] = RN.seederCalibration[fruitName] * (RN.TSM[fruitName]*RN.plantsPerSqm[fruitName])/RN.germination[fruitName];  -- kg/ha
				else
					RN.seedMassRatePerHa[fruitName] = 10000 * RN.seedUsagePerSqm[fruitName];
				end;
				RN.seedVolumeRatePerHa[fruitName] = RN.seedMassRatePerHa[fruitName]/RN.seedDensity[fruitName];
				RN.seedCostPerHa[fruitName] = RN.seedPricePer100kg[fruitName] * RN.seedMassRatePerHa[fruitName]/100;
				RN.seedUsagePerSqm[fruitName] = RN.seedMassRatePerHa[fruitName] / (10000 * RN.seedDensity[fruitName]); -- L/sqm
				RN.cropPricePerLiter[fruitName] = RN.cropDensity[fruitName] * RN.cropPricePer100kg[fruitName]/100;
				RN.seedCostPerLiter[fruitName] = RN.seedDensity[fruitName] * RN.seedPricePer100kg[fruitName]/100;
				
				if RN.harvesterType[fruitName] == "MOWERS" then 
					RN.seedCostPerHa[fruitName] = RN.seedCostPerHa[fruitName];		
				end;

				if Fruits[fruitName] == "MISCANTHUS" then
					RN.seedCostPerHa[fruitName] = RN.seedCostPerHa[fruitName]/20;		-- seed every 20 years.
				end;

--				RN.cropNPKCost[fruitName]	= RN.fertCostPerKg.N * RN.Nitrogen[fruitName]
--											+ RN.fertCostPerKg.P * RN.Phosphate[fruitName]
--											+ RN.fertCostPerKg.K * RN.Potassium[fruitName]; 

-- fertilizer
				local neededN = 0;
				local neededP = 0;
				local neededK = 0;
				
				if RN.Nitrogen[fruitName] ~= nil then
					neededN = RN.Nitrogen[fruitName];
					neededP = RN.Phosphate[fruitName];
					neededK = RN.Potassium[fruitName];
				else
					neededN = RN.Nitrogen["UNDEFINED"];
					neededP = RN.Phosphate["UNDEFINED"];
					neededK = RN.Potassium["UNDEFINED"];
				end;
--[[
				local appliedManure = neededP / RN.PAvailablePerTonne.cowMANURE; -- Ton
				local manureN = appliedManure * RN.NAvailablePerTonne.cowMANURE; -- kg
				local manureP = appliedManure * RN.PAvailablePerTonne.cowMANURE; -- kg
				local manureK = appliedManure * RN.KAvailablePerTonne.cowMANURE; -- kg
				local minFertN = math.max(neededN - manureN,0); -- kg
				local minFertP = math.max(neededP - manureP,0); -- kg
				local minFertK = math.max(neededK - manureK,0); -- kg
				local mineralNApplied = minFertN/0.27;
				local appliedManureCost = appliedManure * 10 * RN.commodityPricePer100kg.MANURE;
				local minFertNCost = minFertN * RN.fertCostPerKg.N;

				local fert1N    = RN.fert1DeliveredN[fruitName];	-- kg/Ha
				local fert1P    = RN.fert1DeliveredP[fruitName];	-- kg/Ha
				local fert1K    = RN.fert1DeliveredK[fruitName]; 	-- kg/Ha
				local fert2N    = RN.fert2DeliveredN[fruitName];	-- kg/Ha
				local fert2P    = RN.fert2DeliveredP[fruitName];	-- kg/Ha						
				local fert2K    = RN.fert2DeliveredK[fruitName];	-- kg/Ha

				local fert1Rate = RN.fert1SprayRate[fruitName];		-- Tonne/Ha
				local fert2Rate = RN.fert2SprayRate[fruitName];		-- Tonne/Ha
				local fertType1 = tostring(RN.fertFirstApplication[fruitName]);
				local fertType2 = tostring(RN.fertSecondApplication[fruitName]);
]]--				
				local fert1Cost = RN.fert1CostPerHa[fruitName];		-- €/Ha
				local fert2Cost = RN.fert2CostPerHa[fruitName];		-- €/Ha
				RN.cropNPKCost[fruitName] = fert1Cost + fert2Cost;
				
-- lime
				local appliedLime = (1/3) * RN.sprayLitersPerHa.LIME * RN.commodityDensity.LIME / 1000; -- Ton, only every third year
				local appliedLimeCost = 10 * appliedLime * RN.commodityPricePer100kg.LIME;
				RN.cropLimeCost[fruitName] = appliedLimeCost;

--				RN.cropNPKCost[fruitName] = appliedManureCost + minFertNCost;

				g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = RN.cropDensity[fruitName]/massScale;
				g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = RN.cropPricePerLiter[fruitName];
				g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = RN.cropPricePerLiter[fruitName];
				g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = RN.cropPricePerLiter[fruitName];					
			end;

			Crop = tostring(fruitName);
			randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * RN.yieldRange[fruitName];
			local randomGrowthFactor = randomYieldFactor;
			bonusYield = bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(RN.cropDensity[fruitName]*10000); --Real life yield in liters/sqm
			baseYield =  gameCorrection *  bonusYield;
			noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
			noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
			LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
			FertilizerBonus = 10000 * baseYield * 0.16665;			-- 16.665% gain
			RN.cropBonusYieldLitersPerHa[fruitName] = 10000 * bonusYield;
			RN.netYield[fruitName]	= RN.cropBonusYieldLitersPerHa[fruitName]
			g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = RN.cropDensity[fruitName]/massScale;
			g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm = gameLevelYieldCorrection * baseYield;
			local hasWindrow = false;
			local windrowName = " ";
			if g_currentMission.fruitTypeManager.fruitTypes[iFruitType].hasWindrow ~= nil then
				hasWindrow = g_currentMission.fruitTypeManager.fruitTypes[iFruitType].hasWindrow;

				if hasWindrow then
					windrowName = g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowName;
					if not(string.find(fruitName, "DRY") or string.find(fruitName, "WET") or string.find(fruitName, "WINDROW") or string.find(fruitName, "FERMENTED")) then
						print("Windrow name : "..windrowName);		-- comment out when not debugging
					end;

					if windrowName == "STRAW" then
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.strawYieldFactor * RN.cropYield[fruitName]*100/(strawDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_WHEAT;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE_WHEAT;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.strawYieldFactor * RN.cropYield[fruitName] * 100;					
						RN.strawBaleIncomePerHa[fruitName] = RN.forageCost.strawSquareBalePrice * RN.numberSquareBalesPerHa[fruitName];	
					elseif windrowName == "RYE_WINDROW" then
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.strawYieldFactor * RN.cropYield[fruitName]*100/(strawDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_WHEAT;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE_WHEAT;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.strawYieldFactor * RN.cropYield[fruitName] * 100;					

					elseif windrowName == "DRYGRASS_WINDROW" then
						bonusWindrow = (1-lossWindrow-lossTedding) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(hayDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "DRYALFALFA_WINDROW" then
						bonusWindrow = (1-lossWindrow-lossTedding) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(hayDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "DRYCLOVER_WINDROW" then
						bonusWindrow = (1-lossWindrow-lossTedding) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(hayDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "DRYLUCERNE_WINDROW" then
						bonusWindrow = (1-lossWindrow-lossTedding) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(hayDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "DRYLUZERNE_WINDROW" then
						bonusWindrow = (1-lossWindrow-lossTedding) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(hayDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "ALFALFA_WINDROW" then
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(silageDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "CLOVER_WINDROW" then
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(silageDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "LUCERNE_WINDROW" then
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(silageDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "LUZERNE_WINDROW" then
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(silageDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "GRASS_WINDROW" and fruitName == "INCARASE_GRASS" then
--						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * 15 * RN.strawYieldFactor * RN.cropYield[fruitName]*100/(silageDensity*10000);
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield["GRASS"]*100/(silageDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_DRYGRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;

					elseif windrowName == "CORN_STALKS" and fruitName == "MAIZE" then				
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_GRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;
						RN.netYield["CHOPPEDMAIZE_FERMENTED"] = RN.windrowYieldLitersPerHa[fruitName];
						RN.netYield["CHOPPEDMAIZE"] = RN.windrowYieldLitersPerHa[fruitName];
						RN.roundBaleVolume["CHOPPEDMAIZE_FERMENTED"] = RN.commodityVolume.ROUNDBALE_GRASS;
						RN.roundBaleVolume["CHOPPEDMAIZE"] = RN.commodityVolume.ROUNDBALE_GRASS;
						RN.squareBaleVolume["CHOPPEDMAIZE_FERMENTED"] = RN.commodityVolume.SQUAREBALE; 
						RN.squareBaleVolume["CHOPPEDMAIZE"] = RN.commodityVolume.SQUAREBALE; 
					elseif windrowName == "HEMP_WINDROW" and fruitName == "HEMP" then				
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_GRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;
						RN.netYield["HEMP_WINDROW"] = RN.windrowYieldLitersPerHa[fruitName];
						RN.roundBaleVolume["HEMP_WINDROW"] = RN.commodityVolume.ROUNDBALE_GRASS;
						RN.squareBaleVolume["HEMP_WINDROW"] = RN.commodityVolume.SQUAREBALE; 						 
					else
						bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName]*100/(silageDensity*10000);
						gameWindrow = gameCorrection * bonusWindrow;
						g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
						RN.windrowYieldLitersPerHa[fruitName] = 10000 * bonusWindrow;
						RN.numberRoundBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.ROUNDBALE_GRASS;
						RN.numberSquareBalesPerHa[fruitName] = RN.windrowYieldLitersPerHa[fruitName]/RN.commodityVolume.SQUAREBALE;
						RN.windrowYieldkgPerHa[fruitName] = (1-lossWindrow) * bonusScale * randomYieldFactor * RN.cropYield[fruitName] * 100;
					end;
					
				end;
			end;
			
			if fruitName == "POPLAR" then
				g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = 0.546 * RN.seedUsagePerSqm[fruitName];
			else
				g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = RN.seedUsagePerSqm[fruitName];
			end;
			
			RN.cropPricePerHa[fruitName] = RN.cropBonusYieldLitersPerHa[fruitName] * RN.cropPricePerLiter[fruitName];
			RN.seedCostPerLiter[fruitName] = RN.seedDensity[fruitName] * RN.seedPricePer100kg[fruitName]/100;

			RN.commoditySeedVolumePerPallet[seedName]	= math.min(1000,10 * math.ceil(RN.HaPerSeedPallet * RN.seedVolumeRatePerHa[fruitName]/10));

			local productCostPerHa = 0;
			
			if RN.harvesterType[fruitName] == "MOWERS" then
--				print(fruitName);
--				print(RN.seedCostPerHa[fruitName]);
--				print(RN.cropNPKCost[fruitName]);
--				print(RN.cropLimeCost[fruitName]);
--				print(RN.cropProtectionCost[fruitName]);
				productCostPerHa = RN.seedCostPerHa[fruitName]/5 + RN.cropNPKCost[fruitName] + RN.cropLimeCost[fruitName] + RN.cropProtectionCost[fruitName]/5; 							 		
			else
				productCostPerHa = RN.seedCostPerHa[fruitName] + RN.cropNPKCost[fruitName] + RN.cropLimeCost[fruitName] + RN.cropProtectionCost[fruitName]; 			
			end;
			 
			RN.productCostPerHa[fruitName] = productCostPerHa;

			if not(string.find(fruitName, "DRY") or string.find(fruitName, "WET") or string.find(fruitName, "WINDROW") or string.find(fruitName, "FERMENTED")) then

				if isDebug then
					myPrintFloat(Crop.." massPerLiter",RN.cropDensity[fruitName],textWidth," kg/L");
					myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
					myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
					myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
					myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
					myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
					myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
					myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
					myPrint(Crop.." max yield",RN.cropBonusYieldLitersPerHa[fruitName],textWidth," L/Ha");

					if hasWindrow then myPrint(Crop.." windrowLiterPerHa",10000* bonusWindrow,textWidth," L/Ha"); end;
					if hasWindrow then myPrintFloat(Crop.." round bales Per Ha",RN.numberRoundBalesPerHa[fruitName],textWidth," Bales/Ha"); end;
					if hasWindrow then 
						myPrintFloat(Crop.." square bales Per Ha",RN.numberSquareBalesPerHa[fruitName],textWidth," Bales/Ha"); 
						windrowName = g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowName;
					end;
					myPrintFloat(Crop.." seed Mass Per Liter",RN.seedDensity[fruitName],textWidth," kg/L");
					myPrintFloat(Crop.." seed Mass Per Ha",RN.seedMassRatePerHa[fruitName],textWidth," kg/Ha");
					myPrintFloat(Crop.." seed Volume Per Ha",RN.seedVolumeRatePerHa[fruitName],textWidth," L/Ha");
					myPrintFloat(Crop.." seed Cost Per Liter",RN.seedCostPerLiter[fruitName],textWidth,currency.."/L");
					myPrintFloat(Crop.." seed Cost Per Ha",RN.seedCostPerHa[fruitName],textWidth,currency.."/Ha");	
					myPrintFloat(Crop.." Lime Cost Per Ha (1/3)",RN.cropLimeCost[fruitName],textWidth,currency.."/Ha");				
					myPrintFloat(Crop.." NPK Cost Per Ha",RN.cropNPKCost[fruitName],textWidth,currency.."/Ha");
					myPrintFloat(Crop.." Crop protection CostPerHa",RN.cropProtectionCost[fruitName],textWidth,currency.."/Ha");		
					myPrintFloat(Crop.." Supply costs Per Ha",productCostPerHa,textWidth,currency.."/Ha");		
					myPrint(Crop.." price Per 1000 Liter",1000*RN.cropPricePerLiter[fruitName],textWidth,currency.."/1000 L");
					myPrint(Crop.." Gross income per ha - no loss",RN.cropPricePerHa[fruitName],textWidth,currency.."/Ha");				
					myPrint(Crop.." Net income per ha - no loss",RN.cropPricePerHa[fruitName]-productCostPerHa,textWidth,currency.."/Ha");
					if windrowName == "STRAW" then
						myPrint(Crop.." Straw-bale income per ha",RN.strawBaleIncomePerHa[fruitName],textWidth,currency.."/Ha");
					end;
					if RN.harvesterType[fruitName] == "MOWERS" then 
						print("Supply costs averaged over 5 years for grass, clover and alfalfa")
					end;			
					print(" ");
				end;
			end;
			
	end;
end;
	 

function RealNumbersCropYield:keyEvent(unicode, sym, modifier, isDown)
end;
function RealNumbersCropYield:update(dt)
end;
function RealNumbersCropYield:draw()
end;
function RealNumbersCropYield:deleteMap()
end;
function RealNumbersCropYield:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersCropYield);
