--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersForestry by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.2.3.9 Compatible with Seasons and Precision Farming
-- 30.11.2018 - 02.01.2021

--[[

This script adjusts the sell price of different types of tree logs. The input parameters
are defined in a Tree logging section in the script RealNumbersInitialization.lua

The user should adjust the prices of saw logs to local pricing levels.

By searching the internet many sources will give different price information.

Stumpage:

	The price you get for a standing tree. The buyer does the felling and hauling. This is
	not the way it can be done in Farming Simulator.

Roadside sawlogs:

	The price you get for roundwood logs picked up at the roadside. You do the felling, the buyer
	picks up the logs from roadside stacks.

Sawlogs:

	Roundwood logs cut to uniform length. Has bark.

Lumber:

	Processed logs, cut into boards.

Board foot:

	The unit of volume of wood in the wood trading business.
	1 BFT = 1 square foot x 1 inch = 2.35973722 liters.
	1 MBFT = 1000 BFT = 2.35973722 cubic meters.

	1 MBFT = 5 - 7 Tonnes soft wood (pine, spruce, poplar)
	1 MBFT = 7 - 9 Tonnes hard wood (ash, beech, birch, larch, locust, mahogany, maple, oak)

Unit conversion

	Logging companies and sawmills often gives prices per MBFT.
	Goverments often give prices per Tonnes or per cubic meter (m3).

	In this mod, input price level is given per MBFT, as this is the most readily available
	information on the internet. To convert to prices per Tonnes and per m3, we need to know
	some density values.

	MBFT is a volume unit. It is a measure of how much cut board volume you get out of a given
	mass of wood. This varies between hard wood and soft wood trees.

	It takes 5 - 7 Tonnes of soft wood tree to produce 1 MBFT cut boards.
	It takes 7 - 9 Tonnes of hard wood tree to produce 1 MBFT cut boards.

	Hence we can convert from currency/MBFT to currency/Tonnes as follows:

	Price/Tonnes = (Price/MBFT)  / (Tonnes/MBFT)

	To convert from price per Tonnes to price per cubic meter we need the mass density (kg/L)
	We then convert from currency/Tonnes to currency/m3 as follows:

	Price/m3 = (Price/Tonnes) * (kg/L)

Delivered logs

	In Farming Simulator, the seller delivers the logs to the sawmill. As default the hauling fee
	is set to 10 $-£-€/Tonnes

	This means that the price for delivered sawlogs is calculated as follows:

	Price/m3 (delivered) = ( (Price/Tonnes) + (Delivery fee/Tonnes) ) * (kg/L)

--]]

RealNumbersForestry = {};

function RealNumbersForestry:loadMap()

	local textWidth = 50;
	local textWidth2 = 15;
	local currency ="€";
	local randomness = 1; 			--1: on, 0: off

	RNS.notRunYetRealNumbersForestry = true;
	RNS.upStartRealNumbersForestry = true;

	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;

	print("\nFS19_RealLifeNumbers: RealNumbersForestry active\n")

	print("MBFT : 1000 board feet = 1000 square feet x 1 inch thick = 2.35973722 cubic meters");
	print("Tonnes per MBFT is how many tonnes of barked tree logs is needed to produce 2.36 cubic meters of cut boards");
	print("Price per MBFT is the input price. Adjust this in RealNumbersInitialization");
	print("Random price variation is used to simulate variation in log quality");
	print("Random price variation can be adjusted in RealNumbersInitialization.lua");
	print("Random price variation can be turned off in RealNumbersForestry.lua");
	print(" ");

-- create a list of trees on the map and save in a table called treeList

	local treeList = {};
	for i,v in pairs(g_splitTypeManager.typesByIndex) do
		treeList[i] = string.upper(v.name);
	end;


-- sort treeList alphabetically and save in sortedTrees

	local sortedTrees = {};
	for i,n in pairs(treeList) do table.insert(sortedTrees,n); end;
	table.sort(sortedTrees);
	RN.sortedTrees = sortedTrees;
	for k,treeName in ipairs(sortedTrees) do
		print(treeName)				-- treeName are names of trees defined on the map.



--	for k,tree in pairs(g_splitTypeManager.typesByIndex) do
--		local treeName = string.upper(tree.name);

		if string.find(RN.loggingTrees.names,treeName) ~= nil then
			RN.treeRandomPriceFactor[treeName] = 1 + randomness * RN.treeLogPriceVariation[treeName] * (math.random()-0.5)/100;
			RN.logsPricePerMBFT[treeName] = RN.treeRandomPriceFactor[treeName] * RN.logsPricePerMBFT[treeName];
			RN.logsPricePerTonnes[treeName] = RN.logsPricePerMBFT[treeName]/RN.woodMBFT[treeName];
			RN.logsPricePerLiter[treeName] = RN.logsPricePerTonnes[treeName] * RN.woodDensity[treeName]; 	-- per 1000 L
			RN.logDeliveredPricePerLiter[treeName] = (RN.logsPricePerTonnes[treeName] + RN.woodDelivery[treeName]) * RN.woodDensity[treeName]; -- per 1000 L
			myPrintFloat(treeName.." : wood mass density ",RN.woodDensity[treeName],textWidth,"kg/L")
			myPrintFloat(treeName.." : Tonnes per 1000 board feet ",RN.woodMBFT[treeName],textWidth,"Tonnes/MBFT")
			myPrint(treeName.." : Delivery ",RN.woodDelivery[treeName],textWidth,currency.."/Tonnes")
			myPrintFloat(treeName.." : RandomPriceFactor",RN.treeRandomPriceFactor[treeName],textWidth," ");
			myPrint(treeName.." : Price per MBFT - roadside log ",RN.logsPricePerMBFT[treeName],textWidth,currency.."/MBFT");
			myPrint(treeName.." : Price per Tonnes - roadside log ",RN.logsPricePerTonnes[treeName],textWidth,currency.."/Tonnes");
			myPrint(treeName.." : Price per 1000 liters - roadside log ",RN.logsPricePerLiter[treeName],textWidth,currency.."/1000 L");
			myPrint(treeName.." : Price per 1000 liters - delivered log ",RN.logDeliveredPricePerLiter[treeName],textWidth,currency.."/1000 L");
			myPrintFloat(treeName.." : Woodchips per liter ",RN.woodChipsPerLiter[treeName],textWidth,"Per Liter")
		else
			print(treeName.." is not defined in FS19_RealLifeNumbers");
		end;
		print(" ");
	end;
end;


function RealNumbersForestry:keyEvent(unicode, sym, modifier, isDown)
	local currency = "€";	-- or  "$" or "£"
    local textWidth1 = 40; 	-- text width in log file
	local textWidth = 50;
	local textWidth2 = 15;

	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end
	if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_q) then
		print(" ");
		print("\nlAlt-rq pressed:  RealNumbersForestry.lua\n");

		print("MBFT : 1000 board feet = 1000 square feet x 1 inch thick = 2.35973722 cubic meters");
		print("Tonnes per MBFT is how many tonnes of barked tree logs is needed to produce 2.36 cubic meters of cut boards");
		print("Price per MBFT is the input price. Adjust this in RealNumbersInitialization");
		print("Random price variation is used to simulate variation in log quality");
		print("Random price variation can be adjusted in RealNumbersInitialization.lua");
		print("Random price variation can be turned off in RealNumbersForestry.lua");
		print(" ");

-- create a list of trees on the map and save in a table called treeList

		local treeList = {};
		for i,v in pairs(g_splitTypeManager.typesByIndex) do
			treeList[i] = string.upper(v.name);
		end;


-- sort treeList alphabetically and save in sortedTrees

		local sortedTrees = {};
		for i,n in pairs(treeList) do table.insert(sortedTrees,n); end;
		table.sort(sortedTrees);
		RN.sortedTrees = sortedTrees;
			for k,treeName in ipairs(sortedTrees) do
				print(treeName)				-- treeName are names of trees defined on the map.




	--	for k,tree in pairs(g_splitTypeManager.typesByIndex) do
	--		local treeName = string.upper(tree.name);

			if string.find(RN.loggingTrees.names,treeName) ~= nil then

				RN.logsPricePerTonnes[treeName] = RN.logsPricePerMBFT[treeName]/RN.woodMBFT[treeName];
				RN.logsPricePerLiter[treeName] = RN.logsPricePerTonnes[treeName] * RN.woodDensity[treeName]; 	-- per 1000 L
				RN.logDeliveredPricePerLiter[treeName] = (RN.logsPricePerTonnes[treeName] + RN.woodDelivery[treeName]) * RN.woodDensity[treeName]; -- per 1000 L
				myPrintFloat(treeName.." : wood mass density ",RN.woodDensity[treeName],textWidth,"kg/L")
				myPrintFloat(treeName.." : Tonnes per 1000 board feet ",RN.woodMBFT[treeName],textWidth,"Tonnes/MBFT")
				myPrint(treeName.." : Delivery ",RN.woodDelivery[treeName],textWidth,currency.."/Tonnes")
				myPrintFloat(treeName.." : RandomPriceFactor",RN.treeRandomPriceFactor[treeName],textWidth," ");
				myPrint(treeName.." : Price per MBFT - roadside log ",RN.logsPricePerMBFT[treeName],textWidth,currency.."/MBFT");
				myPrint(treeName.." : Price per Tonnes - roadside log ",RN.logsPricePerTonnes[treeName],textWidth,currency.."/Tonnes");
				myPrint(treeName.." : Price per 1000 liters - roadside log ",RN.logsPricePerLiter[treeName],textWidth,currency.."/1000 L");
				myPrint(treeName.." : Price per 1000 liters - delivered log ",RN.logDeliveredPricePerLiter[treeName],textWidth,currency.."/1000 L");
				myPrintFloat(treeName.." : Woodchips per liter ",RN.woodChipsPerLiter[treeName],textWidth,"Per Liter")
			else
				print(treeName.." is not defined in FS19_RealLifeNumbers");
			end;
			print(" ");
		end;
	end;
end;


function RealNumbersForestry:update(dt)

	local minute = g_currentMission.environment.currentMinute;


	local runMinute = minute % 10; -- remainder
	if runMinute ~= 5 then RNS.notRunYetRealNumbersForestry = true; end;
	if runMinute == 5 or RNS.upStartRealNumbersForestry then
		if RNS.notRunYetRealNumbersForestry then

--			print("RealNumbersForestry:update"); -- comment out when not debugging

			if g_splitTypeManager ~= nil then
				for k,tree in pairs(g_splitTypeManager.typesByIndex) do
					local name = string.upper(tree.name);
					if RN.woodChipsPerLiter[name] ~= nil then
						tree.woodChipsPerLiter = RN.woodChipsPerLiter[name];
					end;
					if RN.logsPricePerLiter[name] ~= nil then
						tree.pricePerLiter = RN.logDeliveredPricePerLiter[name]/1000;
					end;
				end;
			end;

		end;
		RNS.notRunYetRealNumbersForestry = false;
		RNS.upStartRealNumbersForestry = false;
	end;
end;
function RealNumbersForestry:draw()
end;
function RealNumbersForestry:deleteMap()
end;
function RealNumbersForestry:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersForestry);

