--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersSeeding by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.2.3.9 Compatible with Seasons and Precision Farming
-- 30.11.2018 - 02.01.2021

-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE COW_TYPE_BRAHMAN_BROWN
-- COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

-- Original seeds

-- SEED_BARLEY SEED_CANOLA SEED_MAIZE SEED_MILLET SEED_OAT SEED_OILSEEDRADISH
-- SEED_POTATO SEED_RYE SEED_SOYBEAN SEED_SPELT SEED_SUGARBEET SEED_SUNFLOWER SEED_WHEAT
-- SEED_ALFALFA SEED_CLOVER SEED_GRASS

-- Maybe future seeds

-- SEED_ASPARAGUS  SEED_BEAN SEED_BEANS SEED_BLACKBEANS SEED_CABBAGE
-- SEED_CARROT SEED_COFFEE SEED_COTTON SEED_CRANBERRY SEED_HEMP
-- SEED_HOPS SEED_LETTUCE  SEED_MUSTARD SEED_ONION
-- SEED_PEANUT SEED_PEAS SEED_POPPY SEED_REDCABBAGE
-- SEED_RICE SEED_SORGHUM   SEED_SUDANGRASS
-- SEED_TOBACCO SEED_TRITICALE  SEED_WHITECABBAGE

--[[

	Version 1.1 is a complete redesign of the code. It now aims to be used in cooperation with the mod VariableSprayUsage, which
	allows the user to adjust the spray rate wen driving in the fields. Together, the two mods allow the user to apply spray
	types at different rates to different crops.


Crop nutrient need

	The strategy of this script is to add as much nutrient as is taken up by the crops and removed from the field at harvest.
	A crop's uptake of nutrients (nitrogen (N), phospate (P), and potassium (K)) depends on the yield of each crop. Data for nutrient
	uptake is taken from a USDA database, where the uptake per mass is dependent on moisture content in the crop. Linear regression
	was used to obtain a moisture scaling model, so that the mod user can change the moisture content of the crop, if so desired.
	Many countries, but not all, require N, P and K to be listed in specific forms, such as N, P2O5, and K2O. This convention is used
	in this script.

	The script writes a table to the log file with the NPK needs for each defined fruit. If a map has extra fruits, which have not
	been defined in RealNumbersInitializtion, their NPK needs will an average need.

NPK content in organic fertlizers

	If one were to use organic fertilizers (slurry, manure, and digestate) to cover the total need for NPK replacement, one would
	need many tank loads per hectar. For this reason a parameter (nutrient.organicFertilizerFraction) is used to split the applied
	fertilizer between organic and artificial. The default split is 30% organic and 70% artificial. This plit ratio can be changed
	by the mod user. This approach implies that the first application of NPK is by an organic source and the second by an artificial
	fertilizer.

	The NPK content of organic fertilizers varies. In FS we have five types available: pig slurry, cow slurry, pig manure, cow manure,
	and digestate (from silage fermentation). For slurry and manure, the mod user can set a mixing ratio betwwen pig and cow wastes:


	If your slurry and manure productions are dominated by one type of animal, you can change the two fractions. Information about
	your annual slurry and manure production is provided in log print-outs from another script: RealNumbersAnimalCare.

	Far from all nutrients in a fertilizer is aborbed by the crop. The uptake fraction varies both between the nutients (N, P, K) and between
	the types and sources of organic fertilizers (slurry/manure, pig/cow). This is included in the script by parameters as shown here:

	nutrient.pigSlurryNCropAvailability = 0.55;
	nutrient.pigSlurryPCropAvailability = 0.50;
	nutrient.pigSlurryKCropAvailability = 0.90;
	nutrient.cowManureNCropAvailability = 0.10;
	nutrient.cowManurePCropAvailability = 0.60;
	nutrient.cowManureKCropAvailability = 0.90;

	It is seen that the N utilization from cow manure is particularly low, due to gasseous nitrogen release to the air.

	Using the mixing ratios and the crop-availablility paramters, the script defines the NPK content in a representative slurry/manure.

Nutrients delivered

	The script writes a table to the log file, listing how much of N, P, and K is needed for each fruit type. FS19 does not have
	individual sources of N, P, and K, hence a mix is required. A weight average application need is therefore also printed. The
	cost for each fertilizer type per hectare is also printed, assuming the weighted application rate. The user can adjust the weights
	to make the mix more influenced by one of the three nutrients.

	The amount of liquid fertilizer needed is about half of that for solid fertilizer, as the nutrients come in closer contact with the plants.
	This is included in the script using the parameter (liqfert.liquidReductionFactor = 2).

	Liquid fertilizers can be diluted with water for the sprayer to work effectively at very low nozzle flow rates. The spray rate parameter
	must take this into account.
	The script provides a parameter (liqfert.dilutionFactor) to set the dilution factor. From ver. 1.1.3 it is set so that 1 Liter of liquid fertilizer is
	diluted to a 1 Liter solution (no dilution).

Lime application

	Lime is spread to raise the pH of the soil. How much to spread depends on (1) how much the pH needs to be rised, (2) on the soil type,
	and (3) on the lime source.

	The script allows the user to define these parameters:

	soil.Type = 1;				 		-- 1: Sands and loamy sands: 2: Sandy loams and silt loam; 3: Clay loams and clay
	soil.currentpH = 6;
	soil.targetpH = 6.7;
	lime.NV = 50;  				 		-- (NV = Neutralizing Value) concentration of calcium in lime material, 0 - 100

Herbicide application

	Herbicides are sprayed to kill weeds. In this script, the default amount is 2 kg/ha, but this can be changed by the mod user.
	Herbicides are usually diluted with water and sprayed with a liquid fertilizer sprayer. In the script the default dilution is set
	by the parameter:

	herbicide.dilutionFactor = 200;		-- 1 liter of herbicide to 200 liter of water

				A (L/ha)	B (L/ha)	WATER (L/ha)	Moisture	Time
	Asparagus:	1.5			4.0			200-250			<30%		2-3 weeks before harvest
	Barley		1.5			4.0			 80-250			<30%		2-3 weeks before harvest
	Beans		1.5			3.0			 80-250			<30%		1 week before harvest
	Canola		1.5			4.0			200-250			<30%		2-3 weeks before harvest
	Mustard		1.5			4.0			200-250			<30%		1-2 weeks before harvest
	Oats		1.5			4.0			 80-250			<30%		2-3 weeks before harvest
	Onions		1.5			4.0			200-250			<30%		2-3 weeks before harvest
	Peas		1.5			3.0		 	 80-250			<30%		1 week before harvest
	Potato		1.5			4.0			200-250			<30%		1-2 weeks before harvest
	Sugar beet	1.5			4.0			200-250			<30%		2-3 weeks before harvest
	Wheat		1.5			4.0			 80-250			<30%		2-3 weeks before harvest

	A: Pre emergence of crop.
	B: 1-2 weeks before harvest.
]]--

RealNumbersSeeding = {};

function myPrintSeeding(crop, textWidth, seedRate, palletVolume, palletsPerHa, HaPerPallet)
	local text = crop..string.rep(" ",textWidth-string.len(crop))
	print(string.format("%s: %17.2f %17.0f %17.3f %17.1f", text, seedRate, palletVolume, palletsPerHa, HaPerPallet));
end;




function RealNumbersSeeding:loadMap()
	local currency = " ";
	local isDebug = RN.isDebug;
	local isDebug2 = false;
	local idx = 0;



    local textWidth = 20;
    local textWidth2 = 27;
	local textWidth3 = 40;

	local Fruits = RN.fruits;

	RN.seedTypes = {};
	RN.seedTypesFruitName = {};

	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;


	print("FS19_RealLifeNumbers: RealNumbersSeeding active")
	print("--In-game: press lAlt-ro")

	if isDebug then
		print("\nAmount of seed used per hectare. Useful information when buying seed pallets.");
--		print("Extra crops will have zero values if NPK needs are not defined in RealLifeNumbersInitialization.lua\n");
	end;

	local jj = 0;
	for i,fruitName in ipairs(RN.sortedFruits) do

		local seedName = tostring("SEED_"..fruitName);
--		print(seedName);
--		print(fruitName);
--		if RN.commodityName[seedName] ~= nil then
		if RN.seedTypesInMod[seedName] ~= nil then
			jj = jj + 1;
--			print("seedName is defined");
			RN.seedTypes[jj] = seedName;
			RN.seedTypesFruitName[jj] = tostring(fruitName);
		else
			if not(string.find(seedName, "DRY") or string.find(seedName, "WET") or string.find(seedName, "WINDROW") or string.find(seedName, "FERMENTED")) then
				print(seedName.." is not defined");
			end;
		end;


	end;
	print(" ");
	
	print("Seed type                     Seed rate (L/ha) Pallet volume (L)    Pallets/ha         Ha/pallet\n");

	for i = 1,#RN.seedTypes do
		local seedName = RN.seedTypes[i];
--		local fruitName = tostring(RN.seedTypesFruitName[i]);
		local fruitName = string.sub(seedName,6);
		local seedRate = RN.seedVolumeRatePerHa[fruitName];
		local palletVolume = RN.commoditySeedVolumePerPallet[seedName]
		local palletsPerHa = seedRate/palletVolume;
		myPrintSeeding(fruitName, textWidth, seedRate, palletVolume, palletsPerHa, 1/palletsPerHa)
	end;
	print(" ");
end;

function RealNumbersSeeding:keyEvent(unicode, sym, modifier, isDown)
	local isDebug = RN.isDebug;
	local textWidth = 20;

	if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_o) then
		print("\nlAlt-ro pressed: RealNumbersSeeding\n");

		print("Seed type                     Seed rate (L/ha) Pallet volume (L)    Pallets/ha         Ha/pallet\n");

		for i = 1,#RN.seedTypes do
			local seedName = RN.seedTypes[i];
--		local fruitName = tostring(RN.seedTypesFruitName[i]);
		local fruitName = string.sub(seedName,6);
			local seedRate = RN.seedVolumeRatePerHa[fruitName];
			local palletVolume = RN.commoditySeedVolumePerPallet[seedName]
			local palletsPerHa = seedRate/palletVolume;
			myPrintSeeding(fruitName, textWidth, seedRate, palletVolume, palletsPerHa, 1/palletsPerHa)
		end;
		print(" ");
	end;
end;
function RealNumbersSeeding:update(dt)
end;
function RealNumbersSeeding:draw()
end;
function RealNumbersSeeding:deleteMap()
end;
function RealNumbersSeeding:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersSeeding);
